#!/usr/bin/env python2.3

from random import shuffle
from search import Boggle
from pygame_events import App, EventListener
from pygame_ext import Color, draw_rounded_rect, draw_text
from text import ScrollPane, Button
import pygame

# Define the boggle dice:
default_dice = [
  list('QHIMNU'), list('AOOTTW'), list('ACHOPS'), list('DEILRX'),
  list('AAEEGN'), list('CMTIOU'), list('WHTREV'), list('EENIUS'),
  list('WEEHNG'), list('SSTIOE'), list('OOBBAJ'), list('IDTSYT'),
  list('PFSFAK'), list('RNHLNZ'), list('LYRDEV'), list('TYTREL')]

# Fix english language, change Q to QU %-) 
#default_dice[0][0] = 'QU'


def shake(dice):
  for die in dice:
    shuffle(die)
  shuffle(dice)
  return dice

def first(die):
  return die[0]

def display(board):
  display_row(board[0:4])
  display_row(board[4:8])
  display_row(board[8:12])
  display_row(board[12:16])

def display_row(row):
  print '%2s %2s %2s %2s' % tuple(row)

def format_row(row):
  return '<tr><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>' % tuple(row)
  
def roll():
  return ''.join(map(first, shake(default_dice)))
  
def word(w):
  if w.find('Q'):
    w = w.replace('Q', 'QU')
  return w.lower()
  
def play():
  board = roll()
  b = Boggle()
  b.solve(board)
  if 'Q' in board:
    idx = board.find('Q')
    board = list(board)
    board[idx] = 'Qu'
  solution = map(word, b.found.keys())
  solution.sort()
  return board, solution
  
def text_display():
  board, solution = play()
  display(board)
  print solution

def web_display():
  board, solution = play()
  print 'Content-type: text/html'
  print ''
  print '<html><head>'
  print '<style type="text/css">'
  print 'div{align:center;float:left;}'
  print 'td{font-size:750%;border:solid black 1px;'
  print '  text-align:center;vertical-align: bottom;}'
  print '</style>'
  print '<title>Boggle</title></head><body>'
  print '<div><table>'
  print format_row(board[0:4])
  print format_row(board[4:8])
  print format_row(board[8:12])
  print format_row(board[12:16])
  print '</table></div>'
  print '<div style="float:right;">'
  print '<form style="display:block;" onsubmit="return false;" action="">'
  print '<button style="clear:all;" onclick="document.getElementById(\'solution\').style.visibility = \'visible\'">Show  %s Words</button>' % len(solution)
  print '</form>'
  print '<h2>Solution</h2>'
  print '<div id="solution" style="visibility:hidden;height:400px;width:150px;overflow:auto;">'
  for word in solution:
    print '<span><a target="definition" href="http://www.google.com/search?q=define:%(word)s&ie=UTF-8&oe=UTF-8">%(word)s</a></span><br />' % {'word': word}
  print '</div>'
  print '</div>'
  print '</body></html>'

class BoggleWorld:
  def __init__(self, surface):
    self.surface = surface
    self.surface.fill(Color.white)
    self.rect = self.surface.get_rect()
    self.size = self.width, self.height = self.rect.size
    self.display_area = self.rect.size
    self.center = self.width * 0.5, self.height * 0.5
    self.listener = BoggleListener(self)
    self.line_height = 38
    self.font = pygame.font.Font(None, 72)
    alpha = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
    self.letters = dict(zip(alpha, [self.render_letter(l) for l in alpha]))
    self.letters['Qu'] = self.letters['Q']
    self.new_game()

  def render_letter(self, letter):
    if letter == 'Q': letter = 'Qu'
    msg = self.font.render(letter, 1, Color.black)
    rect = msg.get_rect()
    return msg, rect

  def new_game(self):
    self.board, self.solution = play()
    print 'solution:', self.solution
    holder_rect = self.draw_letters()
    answer_rect = pygame.Rect(holder_rect.right + 30, holder_rect.top, 200, holder_rect.height)
    answer_scroll_height = (len(self.solution) + 2) * self.line_height
    self.answer_scroll = ScrollPane(answer_rect, answer_scroll_height)
    self.answer_scroll.draw(self.surface)
    self.answer_button_rect = pygame.Rect(answer_rect.left, answer_rect.bottom + 10, answer_rect.width, 40)
    self.answer_button = Button(self.answer_button_rect, 'Show Answers')
    self.answer_button.draw(self.surface)
    pygame.display.update()

  def show_answer(self):
    origin = [20,20]
    for answer in self.solution:
      rect = draw_text(self.answer_scroll.surface, Color.black, origin, answer, 36)
      origin[1] += self.line_height
    self.answer_scroll.draw(self.surface)
    pygame.display.update()

  def draw_letters(self):
    holder = self.rect.inflate(-200, -200)
    holder.width = holder.height
    holder.center = self.center
    draw_rounded_rect(self.surface, Color.wheat3, holder, Color.rosybrown2, 5, 20)
    slot_rect = pygame.Rect(0,0,holder.height / 4.0, holder.height / 4.0)
    die_rect = slot_rect.inflate(-20, -20)
    for i in range(4):
      for j in range(4):
        slot_rect.left = holder.left + slot_rect.width * i
        slot_rect.top = holder.top + slot_rect.height * j
        die_rect.center = slot_rect.center
        draw_rounded_rect(self.surface, Color.ivory, die_rect, Color.lightsteelblue, 2, 10)
        letter = self.board[i * 4 + j]
        tile, tile_rect = self.letters[letter]
        tile_rect.center = die_rect.center
        self.surface.blit(tile, tile_rect)
    return holder

  def onclick(self, button, pos):
    if self.answer_button_rect.collidepoint(*pos):
      self.show_answer()

  def scroll_up(self, mod):
    # when adding player's word list, scroll it when shift is held
    self.answer_scroll.scroll_up()
    self.answer_scroll.draw(self.surface)
    pygame.display.update()

  def scroll_down(self, mod):
    self.answer_scroll.scroll_down()
    self.answer_scroll.draw(self.surface)
    pygame.display.update()

  def page_up(self, mod):
    self.answer_scroll.page_up()
    self.answer_scroll.draw(self.surface)
    pygame.display.update()

  def page_down(self, mod):
    self.answer_scroll.page_down()
    self.answer_scroll.draw(self.surface)
    pygame.display.update()

class BoggleListener(EventListener):

  def onnewgame(self):
    self.owner.new_game()

  def onclick(self, button, pos):
    self.owner.onclick(button, pos)

  def onkeypress(self, key, mod, isrepeat):
    if key == pygame.K_UP:
      self.owner.scroll_up(mod)
    elif key == pygame.K_DOWN:
      self.owner.scroll_down(mod)
    elif key == pygame.K_PAGEUP:
      self.owner.page_up(mod)
    elif key == pygame.K_PAGEDOWN:
      self.owner.page_down(mod)
    # if key is alphabetic, add to current word
    # if key is Enter, test, add to word list, and clear current word

def pygame_display():
  app = App()
  app.add_world(BoggleWorld(app.surface))
  app.run()
  

if __name__ == '__main__':
  #web_display() 
  #play()
  pygame_display()
