'''
Demo for Vancouver Python Workshop
'''

import pygame
from geometry import *
from pygame_events import App, ForwardingEventListener, EventDispatcher
from solipong import PongWorld
from goboard import GoWorld
from quipu import HexWorld
from fayray import SpriteWorld
from boggle import BoggleWorld
from kutia import TurtleWorld
from text import SlideWorld, CodeWorld

class DemoDispatcher(EventDispatcher):
  def __init__(self, app):
    EventDispatcher.__init__(self)
    self.app = app

  def _keydown(self, event, time):
    '''
    We want to switch games on Cmd-RightArrow, Cmd-LeftArrow
    '''
    if event.mod & pygame.KMOD_META:
      if event.key == pygame.K_RIGHT:
        self.app.next_world()
      elif event.key == pygame.K_LEFT:
        self.app.prev_world()
    EventDispatcher._keydown(self, event, time)

opening_notes = '''PyGame

    Implementing the big ideas
    from little developers
    



    Dethe Elza. BlastRadius
    This presentation and library available at:
    http://livingcode.sf.net/
    '''

pong_notes = '''Solipong

    The world's simplest game, made simpler
    (turns out to be horribly complex)
    
    (at least with my math skills)
    '''

misc_notes = '''There are lots of tools to help:

    Pd for sound and graphics
    POVRay for nice raytraced bitmaps

Python is fast enough for most purposes

    The nice thing about building several very
    different games is seeing what is common 
    between them'''

boggle_notes = '''Silent Boggle

    Originally a cgi webapp 
    For playing boggle after the kids were asleep
    Uses Peter Norvig's AIML library to solve 
      http://aima.cs.berkeley.edu/python/readme.html

    Dictionary is modified /usr/dict 
    Would be better with a better dictionary

    Use to help kids learn spelling, vocabulary
    '''

sound_notes = '''Notes on sound

    Algorithmic sound is still hard 
    (if you care about cross-platform)
    This is one area where Java is ahead
    But I'm hopeful about using Pd
    http://crca.ucsd.edu/~msp/Pd_documentation/

    There are good free sound libraries
    http://www.a1freesoundeffects.com/noflash.htm
    '''

gomoku_notes = '''Go Moku

    Started as a command-line program, hard to beat
    Ported to PyGame and became much easier, 
    Could see weaknesses in computer's algorithms
    Brute force lookup is pretty fast
    Board games often translate well to networked 
    or AI opponent games
    Goal is to create a program I can't beat'''

quipu_notes = '''Quipu

    Name is Incan form of knotted "writing"
    Based on Tantrix puzzle
    Started in PyObjC, (very nice vector graphics)
    Besides doing things a physical game can do 
      computer games can modify their own pieces, 
      like moving the paths on the tiles
    Game UI will be based on cursors and affordances
    '''

who_notes = '''Who is this for?
    
    Interested dabblers in gaming in python
    Not specific to Pygame
    More about strategies than code
    '''

framework_notes = '''Pygame framework

    I'm not a Pygame expert, more of a tinkerer
    I've tried building games in:
       Tkinter 
       Pygame
       PyObjC
       wxWidgets
       command-line
       webapps
    Any framework has its learning curve, the easier it is, the
    sooner you hit the wall
    '''

whats_next_notes = '''What's Next?

    Framework for scrolling platform game
    Framework for turn-based tile/map game
    Cross-platform algorithmic sound
    Canvas tool for doing vector work in various toolkits
    '''

kutia_notes = '''Kutia

    Tried to write this for wxWindows 
    Wrote it in Tkinter
    Ported to Pygame
    Vector graphics are great for programmers
    GNOME is moving to SVG for games and themes
    Lazy building and caching are key to performance'''

fay_ray_notes = '''Fay Ray

    Hard to find art, I'm not an artist
    Use your kids as artists, but no crayons!
    Codes is easy to find, but no OSS for art (yet)
    No need to be pixel-perfect 8-bit 16x16 pixmaps
    Could use photos or raytraced art 
    (Photorealism: it's not just for Myst anymore)
    Hard to mix photorealism and cartoons though
    Motion helps simple graphics look better'''

strategy_notes = '''The Strategies
    
    High-level frameworks
    Vector Graphics
    Motion can graphics look better
    Be creative about finding artists
    Minimal UI by contextualizing the cursor
    '''

    

def main(fullscreen=False,show_games=False, show_slides=False, show_code=False):
  app = App(fullscreen)
  app.event_dispatcher = DemoDispatcher(app)
  if show_slides:
    app.add_world(SlideWorld(app.new_surface(), opening_notes))
    app.add_world(SlideWorld(app.new_surface(), framework_notes))
    app.add_world(SlideWorld(app.new_surface(), who_notes))
  if show_code:
    app.add_world(CodeWorld(app.new_surface(), 'geometry.py'))
    app.add_world(CodeWorld(app.new_surface(), 'pygame_ext.py'))
    app.add_world(CodeWorld(app.new_surface(), 'pygame_events.py'))
    app.add_world(CodeWorld(app.new_surface(), 'text.py'))
  if show_games:
    app.add_world(PongWorld(app.new_surface()))
  if show_slides:
    app.add_world(SlideWorld(app.new_surface(), pong_notes))
  if show_code:
    app.add_world(CodeWorld(app.new_surface(), 'solipong.py'))
  if show_games:
    app.add_world(BoggleWorld(app.new_surface()))
  if show_slides:
    app.add_world(SlideWorld(app.new_surface(), boggle_notes))
  if show_code:
    app.add_world(CodeWorld(app.new_surface(), 'boggle.py'))
  if show_games:
    app.add_world(GoWorld(app.new_surface()))
  if show_slides:
    app.add_world(SlideWorld(app.new_surface(), gomoku_notes))
  if show_code:
    app.add_world(CodeWorld(app.new_surface(), 'goboard.py'))
    app.add_world(CodeWorld(app.new_surface(), 'gomoku.py'))
  if show_games:
    app.add_world(HexWorld(app.new_surface()))
  if show_slides:
    app.add_world(SlideWorld(app.new_surface(), quipu_notes))
  if show_code:
    app.add_world(CodeWorld(app.new_surface(), 'quipu.py'))
  if show_games:
    app.add_world(TurtleWorld(app.new_surface()))
  if show_slides:
    app.add_world(SlideWorld(app.new_surface(), kutia_notes))
  if show_code:
    app.add_world(CodeWorld(app.new_surface(), 'kutia.py'))
  if show_games:
    app.add_world(SpriteWorld(app.new_surface()))
  if show_slides:
    app.add_world(SlideWorld(app.new_surface(), fay_ray_notes))
  if show_code:
    app.add_world(CodeWorld(app.new_surface(), 'fayray.py'))
  if show_slides:
    app.add_world(SlideWorld(app.new_surface(), whats_next_notes))
    app.add_world(SlideWorld(app.new_surface(), sound_notes))
    app.add_world(SlideWorld(app.new_surface(), misc_notes))
    app.add_world(SlideWorld(app.new_surface(), strategy_notes))
  app.run()

if __name__ == '__main__':
  import sys
  show_games = False
  show_slides = False
  show_code = False
  fullscreen = False
  if 'fullscreen' in sys.argv:  fullscreen = True
  if 'showgames' in sys.argv: show_games = True
  if 'showslides' in sys.argv: show_slides = True
  if 'showcode' in sys.argv: show_code = True
  main(fullscreen, show_games, show_slides, show_code)

