import sys, pygame, gomoku
from pygame_ext import *
from pygame_events import *

def relative(filepath):
  import os.path
  dir = os.path.dirname(__file__)
  return os.path.join(dir, filepath)

class GoWorld(SimpleWorld):
  def __init__(self, surface):
    SimpleWorld.__init__(self, surface, relative('resources/pebbles.jpg'), (1,1), GoListener())
    gomoku.init()
    self.board = pygame.image.load(relative('resources/go_board.gif'))
    self.board = pygame.transform.scale2x(self.board)
    w = self.board.get_width()
    h = self.board.get_height()
    dX = surface.get_width() / 2 - w / 2
    dY = surface.get_height() / 2 - h / 2
    self.inner_rect = dX, dY, w, h
    surface.blit(self.board, self.inner_rect)
    pygame.display.update(self.inner_rect)
    self.offset = dX, dY
    self.offsetR = -dX, -dY
    self.gameOver = False
    self.playersTurn = True
    self.history = []

  def showText(self, text):
    font = pygame.font.Font(None, 72)
    msg = font.render(text, 1, (0,0,255))
    rect = msg.get_rect()
    self.surface.blit(msg,rect)
    pygame.display.update(rect)

  def mouse_to_board(self, pos):
    pos = translate(pos, self.offsetR)   
    return (pos[0] - 13) / 26, (pos[1] - 13) / 26

  def board_to_world(self, x,y):
    return translate(((x+1)*26, (y+1)*26), self.offset)

  def draw_piece(self, pos, isWhite):
    if isWhite: color = Color.white
    else: color = Color.black
    pygame.draw.circle(self.surface, color, self.board_to_world(*pos), 11, 0)
    pygame.display.update()

  def game_over(self, who):
    self.gameOver = True
    self.showText('%s win!' % who)

  def memoize_move(self, player, x, y):
    self.history.append('%s moves to %s,%s' % (player, x, y))
    #print '%s moves to %s,%s' % (player, x, y)

  def player_move(self, pos):
    if self.gameOver: return
    if not self.playersTurn: return
    x,y = self.mouse_to_board(pos)
    self.memoize_move('player', x, y)
    if gomoku.isLegal(x,y):
      playerWin = gomoku.takeMove(x,y)
      self.draw_piece((x,y), True)
      if playerWin:
        self.game_over('You')
      self.playersTurn = False

  def program_move(self):
    if self.gameOver: return
    if self.playersTurn: return
    x,y,programWin = gomoku.programMove()
    self.memoize_move('program', x, y)
    self.draw_piece((x,y),False)
    self.playersTurn = True
    if programWin:
      self.game_over('I')

class GoListener(EventListener):

  def __init__(self):
    self.owner = None

  def onclick(self, button, pos):
    self.owner.player_move(pos)

  def onpulse(self):
    self.owner.program_move()

  def onexit(self):
    #print '\n'.join(self.world.history), '\n'
    #gomoku.playBoard()
    pass

def main():
  app = App()
  world = GoWorld(app.surface)
  app.add_world(world)
  app.run()
	
if __name__ == '__main__':
  main()
