#!/usr/bin/env python2.3

from random import choice

# Globals

PLAYER = '*'
SYSTEM = 'O'

# A block is any group of 5 consecutive positions, horizontal, vertical, or
# diagonal.  Filling any block with your pieces wins the game.
position_to_blocks = [[] for i in range(361)] # 361 positions, each pointing to 3-20 blocks
block_to_positions = [] # 1020 blocks, each pointing to 5 positions
pos_moves = ['.' for i in range(361)]
pos_weights = [0 for i in range(361)]
block_moves = [[None,None,None,None,None] for i in range(1020)]
block_weights = [0 for i in range(1020)]
possible_moves = [True] * 361


def weigh_block(block_idx):
    block = block_moves[block_idx]
    player_count = block.count(PLAYER)
    system_count = block.count(SYSTEM)
    if not player_count and not system_count:
      weight = 1
#    elif player_count and system_count:
#      weight = 0
    else:
      count = player_count or system_count
      if count > 4:
        weight = player_count * 512 + system_count * 1024
      if count == 4:
        weight = player_count * 256 + system_count * 512
      elif count == 3:
        weight = player_count * 32 + system_count * 64
      elif count == 2:
        weight = player_count * 8 + system_count * 16
      elif count == 1:
        weight = player_count * 2 + system_count * 4
    block_weights[block_idx] = weight
    for pos in block_to_positions[block_idx]:
      weigh_pos(pos)
      
def weigh_pos(pos_idx):
  pos_weights[pos_idx] = sum([block_weights[x] for x in position_to_blocks[pos_idx]])
      
def playBoard():
  print '   ', '  '.join(list('ABCDEFGHIJKLMNOPQRS'))
  for i in range(19):
    print '%2d' % (i,),
    for j in range(19):
      pos = i * 19 + j
      print '%2s' % pos_moves[pos], 
    print '\n',

def board():
  for i in range(19):
    for j in range(19):
      print '%3d' % (i * 19 + j),
    print '\n',

def isLegal(x, y):
  if x < 0 or x > 18: return False
  if y < 0 or y > 18: return False
  pos = x + y * 19
  if pos_moves[pos] == '.': return True

def takeMove(x,y):
  pos = x + y * 19
  takeMoveAny(pos, PLAYER)
  return didPlayerWin(pos)

def programMove():
  pos = selectMove()
  takeMoveAny(pos, SYSTEM) 
  y,x = divmod(pos, 19)
  return x, y, didProgramWin(pos)

def takeMoveAny(pos, player):
  pos_moves[pos] = player
  try:
    possible_moves[pos] = False
  except ValueError, e:
    print 'Error: tried to remove %s from %s' % (pos, list(enumerate(possible_moves)))
    raise
  for block in position_to_blocks[pos]:
    index = block_to_positions[block].index(pos)
    block_moves[block][index] = player
    weigh_block(block)

def didPlayerWin(pos):
  for block in position_to_blocks[pos]:
    if block_moves[block].count(PLAYER) == 5:
      return True
  return False

def didProgramWin(pos):
  for block in position_to_blocks[pos]:
    if block_moves[block].count(SYSTEM) == 5:
      return True
  return False

def move(pos, player):
  print 'called move()'
  if pos_moves[pos] != '.':
    if pos_moves[pos] == PLAYER:
      print "You've already moved there, try another move"
    else:
      print "I've already moved there, try another move"
  else:
    pos_moves[pos] = player
    possible_moves[pos] = False
    for block in position_to_blocks[pos]:
      index = block_to_positions[block].index(pos)
      block_moves[block][index] = player
      if block_moves[block].count(player) == 5:
        if player == PLAYER:
	  print 'You win!'
	else:
	  print 'I win!'
	import sys
	sys.exit()
      else:
        for block in position_to_blocks[pos]:
          weigh_block(block)


def selectMove():
  weights = [pos_weights[i] for i in range(361) if possible_moves[i]]
  max_weight = max(weights)
  probable_moves = [i for i,wt in enumerate(pos_weights) if wt  == max_weight]
  print '%d moves of weight %d:  %s' % (len(probable_moves), max_weight, probable_moves)
  mv = choice(probable_moves)
  #print 'choosing %s from %s with a weight of %d' % (positionToMove(mv),
  #  [positionToMove(x) for x in probable_moves], max_weight)
  return mv

def moveToPosition(move):
  letter = move[0]
  y = int(move[1:])
  x = ord(letter) - 97 # only handles lower-case for now
  return x + y * 19

def positionToMove(pos):
  y,x = divmod(pos, 19)
  x = chr(x + 97)
  return '%s%s' % (x,y)
      
def appendPositions(positions):
  block = len(block_to_positions)
  block_to_positions.append(positions)
  for pos in positions:
    position_to_blocks[pos].append(block)
    

def init():
  # rows
  for i in range(19):
    for j in range(15):
      base = i * 19 + j
      appendPositions([base + w for w in range(5)])
  # columns
  for i in range(15):
    for j in range(19):
      base = i * 19 + j
      appendPositions([base + w * 19 for w in range(5)])
  # diagonals
  for i in range(15):
    for j in range(15):
      base = i * 19 + j
      appendPositions([base + w * 20 for w in range(5)])
  for i in range(15):
    for j in range(15):
      base = i * 19 + j + 4
      appendPositions([base + w * 18 for w in range(5)])


def test():
  while 1:
    query = raw_input('position or block? ')
    if query[0] == 'q': break
    elif query[0] == 'b':
      index = int(query.split()[1])
      try:
        print "block %d: %s" % (index, block_to_positions[index])
      except IndexError:
        print 'Index Error: there are only %d blocks' % len(block_to_positions)
    elif query[0] == 'p':
      index = int(query.split()[1])
      try:
        print "position %d: %s" % (index, position_to_blocks[index])
      except IndexError:
        print 'Index Error: there are only %d positions' % len(position_to_blocks)
    else:
      print '''Usage:
        p [index]             (view position to block mapping)
	b [index]             (view block to position mapping)
	q                     (quit)'''

lastMove = None

def main():
  global lastMove
  while 1:
    playBoard()
    if lastMove:
      print 'I moved at', lastMove
    player_move = raw_input('Your move? ')
    if player_move == 'dump':
      print possible_moves
      sys.exit()
    pos = moveToPosition(player_move)
    move(pos, PLAYER)
    system_move = selectMove()
    lastMove = positionToMove(system_move)
    move(system_move, SYSTEM) 

if __name__ == '__main__':
  init()
  main()
