'''
Pong for one, on a circular board
'''

import pygame
from geometry import *
from pygame_ext import VectorSprite, Color, draw_curve
from pygame_events import App, EventListener

'''
How do I determine the bounding rectangle of an arc?
'''

class PongWorld:

  def __init__(self, surface):
    self.surface = surface
    self.surface.fill(Color.white)
    self.rect = self.surface.get_rect()
    self.size = self.width, self.height = self.rect.size
    self.display_area = self.rect.size
    self.center = self.width * 0.5, self.height * 0.5
    pygame.draw.circle(self.surface, Color.black, self.center, 30)
    pygame.draw.circle(self.surface, Color.grey20, self.center, 20)
    pygame.draw.circle(self.surface, Color.grey40, self.center, 10)
    self.listener = PongEvents(self)
    boardradius = min(self.center[X], self.center[Y]) * 0.8
    self.paddle_radius = self.rect.height / 3
    self.paddle = Paddle(self.center, self.paddle_radius, pi/12, Color.black)
    self.paddle.draw(self.surface)
    self.ball = Ball()
    self.ball.draw(self.surface)

  def paddle_right(self, moveby=3):
    self.paddle.right(self.surface, moveby)
    self.paddle.draw(self.surface, moveby)

  def paddle_left(self, moveby=3):
    self.paddle.left(self.surface, moveby)
    self.paddle.draw(self.surface, moveby)

  def move_ball(self):
    self.ball.move(self.surface)
    self.ball.draw(self.surface)

  def test(self):
    # is ball bouncing from outside of paddle?
    # if ball.position is within self.paddle_radius +/- 10: bounce in
    # if ball.position is within self.paddle_radius + 11 .. + 30: bounce out
    # if ball leaves screen, player loses a point, new ball
    # if ball is within radius 20 of center, player gains point, new ball
    pass

class PongEvents(EventListener):

  def __init__(self, owner):
    EventListener.__init__(self, owner)
    self.moving = True
    self.dir_right = True

  def onclick(self, pos, modifiers):
    self.moving = not self.moving
    #if not self.moving: print self.owner.paddle.paddle_degrees()

  def onkeypress(self, key, mod, isrepeat):
    if key == pygame.K_RIGHT:
       self.moving = True
       self.dir_right = True
    elif key == pygame.K_LEFT:
       self.moving = True
       self.dir_right = False

  def onpulse(self):
    keys = pygame.key.get_pressed()
    if keys[pygame.K_RIGHT]:
      self.owner.paddle_right()
    elif keys[pygame.K_LEFT]:
      self.owner.paddle_left()
    self.owner.move_ball()
    pygame.display.update()
      
angles = [degToRad(i) for i in range(360)]
one_deg = degToRad(1)

class Ball(VectorSprite):

  def __init__(self):
    VectorSprite.__init__(self)
    self.position = [200,20]
    self.direction = Vector(3,5)

  def move(self, surface):
    self.draw(surface, Color.white)
    self.position[0] += self.direction.x
    self.position[1] += self.direction.y

  def bounce(self, normal):
    self.direction = self.direction - 2 * dot(self.direction, normal) * normal

  def draw(self, surface, color=Color.black):
    pygame.draw.circle(surface, color, self.position, 10)

class Paddle(VectorSprite):

  def __init__(self, center, radius, size, color):
    VectorSprite.__init__(self)
    self.center = center
    self.size = size
    self.position = 0
    self.pos_index = 0
    self.width = 20
    self.color = color
    self.radius = radius
    self.rect = pygame.Rect(center[0] - radius, center[1] - radius, radius * 2, radius * 2)
  
  def paddle_radians(self):
    start = self.position - self.size / 2
    end = self.position + self.size / 2
    return start, end

  def paddle_degrees(self):
    start, end = self.paddle_radians()
    return radToDegree(start), radToDegree(end)

  def draw(self, scene, color=None):
    start, end = self.paddle_radians()
    if not color:
      color = self.color
    draw_curve(scene, color, self.center, self.radius, start, end, self.width)

  def right(self, scene, moveby=3):
    self.draw(scene, Color.white)
    self.pos_index = (self.pos_index + moveby) % 360
    self.position = angles[self.pos_index]

  def left(self, scene, moveby=3):
    self.draw(scene, Color.white)
    self.pos_index = (self.pos_index - moveby) % 360
    self.position = angles[self.pos_index]

def main():
  app = App()
  world = PongWorld(app.surface)
  app.add_world(world)
  app.run()
    
if __name__ == '__main__':
  main()
