'''
Scrolling text world, slide world
'''

import pygame
from pygame_ext import draw_text, Color, SimpleWorld, draw_rounded_rect
from pygame_events import App, EventListener

class SlideWorld:

  def __init__(self, surface, text=''):
    self.surface = surface
    self.surface.fill(Color.white)
    self.rect = self.surface.get_rect()
    self.listener = EventListener(self)
    self.text_lines = text.splitlines()
    self.layout()

  def set_text(self, text):
   self.text_lines = text.splitlines()
   self.layout()

  def layout(self):
    no_lines = len(self.text_lines)
    origin = [40,40]
    for line in self.text_lines:
      rect = draw_text(self.surface, Color.black, origin, line, 54)
      origin[1] = rect.bottom + 5
    pygame.display.update()


class ScrollListener(EventListener):
  def onkeypress(self, key, mod, isrepeat):
    if key == pygame.K_DOWN:
      self.owner.scroll_down()
    elif key == pygame.K_UP:
      self.owner.scroll_up()
    elif key == pygame.K_PAGEUP:
      self.owner.page_up()
    elif key == pygame.K_PAGEDOWN:
      self.owner.page_down()

  def ondragbegin(self, pos):
    self.owner.ondragbegin(pos)

  def ondragend(self, pos):
    self.owner.ondragend(pos)

  def ondrag(self, pos):
    self.owner.ondrag(pos)


class CodeWorld:

  def __init__(self, surface, filename):
    self.surface = surface
    self.surface.fill(Color.white)
    self.rect = self.surface.get_rect()
    self.listener = ScrollListener(self)
    self.origin = [40,40]
    f = file(filename)
    self.text_lines = [filename.upper(),''] + f.readlines()
    f.close()
    if self.text_lines:
      self.layout()
    else:
      self.text_lines = []

  def layout(self):
    line_height = 38
    no_lines = len(self.text_lines)
    self.scrollpane = ScrollPane(self.rect, (no_lines + 2) * line_height)
    origin = self.origin[:]
    #self.surface.fill(Color.white)
    for line in self.text_lines:
      rect = draw_text(self.scrollpane.surface, Color.black, origin, line.rstrip(), 36)
      origin[1] = rect.bottom + 5
    self.scrollpane.draw(self.surface)

  def scroll_down(self):
    self.scrollpane.scroll_down()
    self.scrollpane.draw(self.surface)
    pygame.display.update()

  def scroll_up(self):
    self.scrollpane.scroll_up()
    self.scrollpane.draw(self.surface)
    pygame.display.update()

  def page_down(self):
    self.scrollpane.page_down()
    self.scrollpane.draw(self.surface)
    pygame.display.update()

  def page_up(self):
    self.scrollpane.page_up()
    self.scrollpane.draw(self.surface)
    pygame.display.update()

  def ondragbegin(self, pos):
    self.scrollpane.ondragbegin(pos)

  def ondragend(self, pos):
    self.scrollpane.ondragend(pos)

  def ondrag(self, pos):
    self.scrollpane.ondrag(pos)

SCROLL_AMOUNT = 300
SCROLLBAR_WIDTH = 24

class ScrollPane:

  def __init__(self, rect, scrollheight=1200):
    self.rect = pygame.Rect(rect)
    self.content_rect = pygame.Rect(self.rect).inflate(-2,-2)
    self.content_rect.width -= SCROLLBAR_WIDTH
    self.thumb_height = 32
    self.scrollbar_rect = pygame.Rect(self.rect)
    self.scrollbar_rect.width = SCROLLBAR_WIDTH
    self.scrollbar_rect.right = self.rect.right
    self.thumb_rect = pygame.Rect(self.scrollbar_rect)
    self.thumb_rect.height = self.thumb_height
    self.thumb_rect.top = self.scrollbar_rect.top
    self.thumb_rect.inflate(-2,-2)
    self.scroll_rect = pygame.Rect(self.content_rect)
    self.scroll_rect.height = scrollheight
    self.surface = pygame.Surface((self.scroll_rect.width, self.scroll_rect.height))
    self.surface.fill(Color.grey90)
    self.clip = pygame.Rect(self.content_rect)
    self.clip.top = 0
    self.clip.left = 0

  def draw(self, surface):
    surface.lock()
    pygame.draw.rect(surface, Color.grey95, self.rect)
    pygame.draw.rect(surface, Color.grey10, self.rect, 1)
    pygame.draw.rect(surface, Color.grey90, self.scrollbar_rect)
    pygame.draw.rect(surface, Color.grey10, self.scrollbar_rect, 1)
    pygame.draw.rect(surface, Color.grey20, self.thumb_rect)
    pygame.draw.rect(surface, Color.grey10, self.thumb_rect, 1)
    print 'content_rect:', self.content_rect
    print 'clip:', self.clip
    surface.blit(self.surface, self.content_rect, self.clip)
    surface.unlock()

  def ondragbegin(self, pos):
    pass

  def ondragend(self, pos):
    pass

  def ondrag(self, pos):
    pass

  def page_up(self):
    self.clip.top -= 400
    if self.clip.top < 0: self.clip.top = 0


  def page_down(self):
    self.clip.bottom += 400
    if self.clip.bottom > self.scroll_rect.bottom: self.clip.bottom = self.scroll_rect.bottom
    pygame.display.update()

  def scroll_up(self):
    self.clip.top -= 30
    if self.clip.top < 0: self.clip.top = 0
    

  def scroll_down(self):
    self.clip.bottom += 30
    if self.clip.bottom > self.scroll_rect.bottom: self.clip.bottom = self.scroll_rect.bottom

class Button:

  def __init__(self, rect, text, onclick=None):
    self.rect = rect
    self.content_rect = pygame.Rect(rect)
    self.content_rect.top = 0
    self.content_rect.left = 0
    self.content_rect.inflate_ip(-4, -4)
    self.surface = pygame.Surface(rect.size, 0)
    self.surface.fill(Color.white)
    draw_rounded_rect(self.surface, Color.white, self.content_rect, Color.black, 2, 5)
    text = draw_text(self.surface, Color.black, self.content_rect.center,
    text, 36, 'center')

  def draw(self, surface):
    surface.blit(self.surface, self.rect)

class TextWorld(SimpleWorld):

  def __init__(self, surface):
    World.__init__(self, surface, None, (1,1), EventListener())
    scroll1 = ScrollPane((40,40), (200,600), 2400)
    scroll2 = ScrollPane((600, 40), (300,400))
    scroll1.draw(surface)
    scroll2.draw(surface)

def main():
  app = App()
  app.add_world(CodeWorld(app.surface, 'text.py'))
  app.run()

if __name__ == '__main__':
  main()
